/*******************************************************************************

   TIMER PARA ACIONAR MOTOR VIRADOR DE OVOS EM CHOCADEIRAS CASEIRAS


   C/ PIC12F675

   AUTOR: CLAUDIO LRIOS

   DATA: 26/10/2020

   COM TEMPO DE INTERVALO ENTRE VIRADAS REGULVEL DE 1 A 8 HORAS
   E TEMPO DE ACIONAMENTO DO MOTOR DE VIRAGEM REGULVEL DE 1 A 16 SEGUNDOS LIGADO

   PINAGEM DO PIC 12F675:
   1-+5V
   2- XTAL (4MHZ)
   3- XTAL
   4- NC (NO CONECTADO)
   5- POTENCIMETRO DE 10K (REGULA O INTERVALO)
   6- POTENCIMETRO DE 10K (REGULA O TEMPO QUE O MOTOR FICA LIGADO)
   7- SADA PARA ACIONAMENTO DO MOTOR VIA RELE OU TIRISTOR
   8- GND


   FOI UTIIZADO O MOTOR ASSINCRONO AC TIPO 49TYJ (USADO EM PRATOS DE
   MICROONDAS ANTIGOS) PARA MOVIMENTAR OVOS NAS BANDEIJAS DE CHOCADEIRAS.
   A TENSO DESTE MOTOR DEVER SER IGUAL A TENSO DA REDE ELTRICA
   UTILIZADA PELO USURIO DESTA MONTAGEM.
   ESTE MOTOR TEM UMA ROTAO DE 6 RPM OU SEJA, A CADA 10 SEGUNDOS D UMA
   VOLTA COMPLETA NO SEU EIXO DE SADA.
   EM GERAL, SE AJUSTA O TEMPO PARA DAR 1/2 VOLTA (5 SEG.) OU 1/3 VOLTA (3 SEG).

   USO DIDTICO

   USE COMPILADOR CCS C 3.200

   ESTE ARQUIVO   PARTE INTEGRANTE DO BLOG PICSOURCE

******************************************************************************/
#include <12F675.h>
#device adc=8
#use delay(clock=4000000)
#fuses WDT,HS, NOCPD, NOPROTECT, NOMCLR, NOPUT, NOBROWNOUT

#byte GPIO = 0x05
#byte TRISIO = 0x85

//BIT
#bit  motor  = 0x5.0//sada motor do virador via rele ou triac (pino 7)

//variavis globais
unsigned int32 tempo=30000;
unsigned int32 tempo_ant=0;
unsigned int32 cnt32=0;
unsigned int32 tempo_interv=0;
unsigned int16 lido=0;
unsigned int8  tempo_acionado=0, a;
unsigned int1  fl=0, fac=0;
/******************************************************************************
           ROTINA  DE INTERRUPO DO TIMER 0 (CONTA 1 SEGUNDO)
******************************************************************************/
#INT_TIMER0
void INT_TIMER0_ISR(){
//gera 1 segundos com metodo de acumulo de erro
// ajuste o timer para o prescaller igual abaixo no main

 #define presc  64 // valor do prescaller do timer usado (8 bits)
 #define fator  (256*presc*4)
 #define f_xtal 4000000  //frequencia do cristal usado (mesmo do cabealho)

   restart_wdt();//restart do wdt

   cnt32+=fator;
   if(cnt32>=f_xtal){//temos um segundo aqui

     fl=1; //autoriza a leitura dos potencimetros
     cnt32-=f_xtal;//subtrai xtal da contagem
     tempo++;//incrementa contador de tempo mximo
     if(tempo>=tempo_interv){
      if(fac==0){
        tempo_ant=tempo;
        fac=1;
        motor=1;//liga motor

      }
      else{
         if(tempo>(tempo_acionado + tempo_ant)){
           motor=0;//desliga motor
           fac=0;
           tempo=0; //reseta contador principal de tempo           
         }
      }//else

     }//if(tempo>=tempo_interv){
    }// if(cnt32>=f_xtal){
}//void INT_TIMER0_ISR(){





/*******************************************************************************
        FAZ LEITURA DE UM CANAL DO CONVERSOR ANALGICO/DIGITAL
  
      
      Conforme o canal passado para a funo temos as seguintes formulas:
      
      canal 2 - pino 5
      Tempo de intervalo (em horas)= lido X 984375 /10000;
      Com leitura = 0 em 'lido' ainda tem que ter 1 hora de intervalo. Ento 
      somado 3600 segundos a varivel 'tempo_interv'.
      O valor 984375  obtido da diviso de 8 horas (8 x 3600 seg =28800 seg)
      menos 1 hora (3600 seg) e depois dividindo por 256.
      Abaixo podemos ver a 'matemtica' usada:
      
      28800 - 3600 = 25200  //subtrai 1 hora 
      
      25200/256= 98,4375   //divide valor encontrado pelo limite da converso AD
      
      Para trabalhar com nmeros inteiros convertemos 98,4375 em:
      984375/10000 (frao com numeros inteiros)
      
      Canal 1 - pino 6
      Tempo de durao do movimento do motor (motor ligado)
      O valor obtido do potenciomtro de durao  dividido por 16 para obtermos
      uma escala de 1 a 16 segundos no curso do potencimetro de durao.
      Obs. O segundo mnimo  gerado automticamente na rotina de interrupo 
      do timer 0 a cada segundo. Quando atinge o valor para ligar, somente no 
      prximo segundo que a rotina ir comparar o valor do potencimetro de 
      durao, se deve ou no desligar o motor de viragem dos ovos.     
      
*******************************************************************************/
void get_canal(unsigned int8 q){
     set_adc_channel(q); //ajusta para o pino 5 (AN2) ou  6 (AN1)
     delay_us(10);
     lido=0;// l valor da tenso do pino escolhido
     for(a=0;a<16;a++){//soma as 16 leituras sucessivas
     lido+=READ_ADC();
     }
     lido= lido>>4;//mdia das ultimas 16 leituras
     if(q==2){ //le ajuste de tempo de intervalo
      // tempo_interv= (lido * 1125) / 10; // converte para minutos para comparar
      tempo_interv= (lido * 984375) / 10000; // converte para minutos para comparar
       tempo_interv=tempo_interv + 3600;//minimo de tempo = 1hora
     }
     if(q==1){//le ajuste de tempo de motor ligado
       tempo_acionado= lido>>4;//divide por 16
     }
}


/******************************************************************************
           ROTINA  MAIN
******************************************************************************/


void main() {
   setup_adc_ports(sAN2 | VSS_VDD);  //seleciona AN2 e 5v como referncia
   setup_adc(ADC_CLOCK_DIV_8 );// ajusta o clock do adc
   set_adc_channel(2); //ajusta para o pino 5 (AN2)
   setup_comparator(NC_NC_NC_NC);//desliga comparadores internos
   setup_counters(RTCC_INTERNAL,RTCC_DIV_64);//prescaller divide por 64
   setup_timer_1(T1_DISABLED);
   setup_vref(FALSE);//desliga voltagem de referencia
   GPIO=0;//idem
   TRISIO=0b111110;//sentido fixo das portas
   enable_interrupts(int_timer0);//liga interrupo do timer 0
   enable_interrupts(global);//mantm desabilitada a geral

/******************************************************************************
           LOOP PRINCIPAL DE REPETIO
******************************************************************************/

   for(;;){
    if(fl==1){
      get_canal(2);// obtem valor do potencimetro de intervalo (pino 5)
      get_canal(1);// obtem valor do potencimetro de tempo ligado (pino 6)
      fl=0;
    }
    while(fl==0);//aguarda interrupo de segundo


   }//for(;;)

}//void main()
